@@ -46,24 +46,33 @@ class EventsCalendar(Service, HTMLCalendar):
         EVENTCAL_TD_DAY_LINK_TODAY = self.srvbld.template('EVENTCAL_TD_DAY_LINK_TODAY')
         EVENTCAL_TD_DAY_NOLINK = self.srvbld.template('EVENTCAL_TD_DAY_NOLINK')
         EVENTCAL_TD_DAY_NOLINK_TODAY = self.srvbld.template('EVENTCAL_TD_DAY_NOLINK_TODAY')
-        if cal_date in self.events_days[self.year]: # check if current calendar tuple date exist in our list of events days
-            eday = day # if it does exist set the event day var with it
-        if day == 0:
-            return EVENTCAL_TD_NODAY # day outside month
-        elif day == eday:
-            # Check if this is one of the events days.
-            # If yes, return link to page
-            EVENT_PAGE = "events_%4d%02d%02d" % (self.year, self.month, day)
-            EVENT_PAGE_VALID_FNAME = valid_filename(EVENT_PAGE)
-            if self.month == self.now.month and day == self.now.day:
-                return EVENTCAL_TD_DAY_LINK_TODAY % (self.cssclasses[weekday], EVENT_PAGE_VALID_FNAME, day)
+        # check if current calendar tuple date exist in our list of events days
+        try:
+            self.events_days[self.year]
+            if cal_date in self.events_days[self.year]:
+                eday = day # if it does exist set the event day var with it
+
+            if day == 0:
+                return EVENTCAL_TD_NODAY # day outside month
+            elif day == eday:
+                # Check if this is one of the events days.
+                # If yes, return link to page
+                EVENT_PAGE = "events_%4d%02d%02d" % (self.year, self.month, day)
+                EVENT_PAGE_VALID_FNAME = valid_filename(EVENT_PAGE)
+                if self.month == self.now.month and day == self.now.day:
+                    return EVENTCAL_TD_DAY_LINK_TODAY % (self.cssclasses[weekday], EVENT_PAGE_VALID_FNAME, day)
+                else:
+                    return EVENTCAL_TD_DAY_LINK % (self.cssclasses[weekday], EVENT_PAGE_VALID_FNAME, day)
             else:
-                return EVENTCAL_TD_DAY_LINK % (self.cssclasses[weekday], EVENT_PAGE_VALID_FNAME, day)
-        else:
-            if self.month == self.now.month and day == self.now.day:
-                return EVENTCAL_TD_DAY_NOLINK_TODAY % (self.cssclasses[weekday], day)
-            else:
-                return EVENTCAL_TD_DAY_NOLINK % (self.cssclasses[weekday], day)
+                if self.month == self.now.month and day == self.now.day:
+                    return EVENTCAL_TD_DAY_NOLINK_TODAY % (self.cssclasses[weekday], day)
+                else:
+                    return EVENTCAL_TD_DAY_NOLINK % (self.cssclasses[weekday], day)
+        except:
+                if self.month == self.now.month and day == self.now.day:
+                    return EVENTCAL_TD_DAY_NOLINK_TODAY % (self.cssclasses[weekday], day)
+                else:
+                    return EVENTCAL_TD_DAY_NOLINK % (self.cssclasses[weekday], day)
 
     def formatweek(self, theweek):
         """Return a complete week as a table row."""
@@ -93,9 +102,10 @@ class EventsCalendar(Service, HTMLCalendar):
         LINK = self.srvbld.template('LINK')
         dt = guess_datetime("%4d-%02d-01" % (theyear, themonth))
         month_name = datetime.strftime(dt, "%B %Y")
-        if self.ml[theyear][themonth]:
+        try:
+            self.ml[theyear][themonth]
             link = LINK % ("uk-link-heading", "events_%4d%02d.html" % (theyear, themonth), "uk-text-uppercase uk-text-muted", month_name)
-        else:
+        except KeyError:
             # ~ link = LINK % ("uk-link-heading", "", "", month_name)
             link = """<span class="%s">%s</span>""" % ("uk-text-uppercase uk-text-muted", month_name)
         return link
